# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext
from ._AgentDefinition import AgentDefinition
from ._PropertySchema import PropertySchema
from ._Resource import Resource


@dataclass
class AgentManifest:
    """The following represents a manifest that can be used to create agents dynamically.
    It includes parameters that can be used to configure the agent's behavior.
    These parameters include values that can be used as publisher parameters that can
    be used to describe additional variables that have been tested and are known to work.

    Variables described here are then used to project into a prompt agent that can be executed.
    Once parameters are provided, these can be referenced in the manifest using the following notation:

    `{{myParameter}}`

    This allows for dynamic configuration of the agent based on the provided parameters.
    (This notation is used elsewhere, but only the `param` scope is supported here)

    Attributes
    ----------
    name : str
        Name of the manifest
    displayName : str
        Human-readable name of the manifest
    description : Optional[str]
        Description of the agent's capabilities and purpose
    metadata : Optional[dict[str, Any]]
        Additional metadata including authors, tags, and other arbitrary properties
    template : AgentDefinition
        The agent that this manifest is based on
    parameters : PropertySchema
        Parameters for configuring the agent's behavior and execution
    resources : list[Resource]
        Resources required by the agent, such as models or tools
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    name: str = field(default="")
    displayName: str = field(default="")
    description: Optional[str] = None
    metadata: Optional[dict[str, Any]] = None
    template: AgentDefinition = field(default_factory=AgentDefinition)
    parameters: PropertySchema = field(default_factory=PropertySchema)
    resources: list[Resource] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "AgentManifest":
        """Load a AgentManifest instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            AgentManifest: The loaded AgentManifest instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for AgentManifest: {data}")

        # create new instance
        instance = AgentManifest()

        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "displayName" in data:
            instance.displayName = data["displayName"]
        if data is not None and "description" in data:
            instance.description = data["description"]
        if data is not None and "metadata" in data:
            instance.metadata = data["metadata"]
        if data is not None and "template" in data:
            instance.template = AgentDefinition.load(data["template"], context)
        if data is not None and "parameters" in data:
            instance.parameters = PropertySchema.load(data["parameters"], context)
        if data is not None and "resources" in data:
            instance.resources = AgentManifest.load_resources(
                data["resources"], context
            )
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_resources(
        data: dict | list, context: Optional[LoadContext]
    ) -> list[Resource]:
        if isinstance(data, dict):
            # convert simple named resources to list of Resource
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "kind": v})
            data = result
        return [Resource.load(item, context) for item in data]

    @staticmethod
    def save_resources(
        items: list[Resource], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the AgentManifest instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.name is not None:
            result["name"] = obj.name
        if obj.displayName is not None:
            result["displayName"] = obj.displayName
        if obj.description is not None:
            result["description"] = obj.description
        if obj.metadata is not None:
            result["metadata"] = obj.metadata
        if obj.template is not None:
            result["template"] = obj.template.save(context)
        if obj.parameters is not None:
            result["parameters"] = obj.parameters.save(context)
        if obj.resources is not None:
            result["resources"] = AgentManifest.save_resources(obj.resources, context)

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the AgentManifest instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the AgentManifest instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class ModelOptions:
    """Options for configuring the behavior of the AI model.
    `kind` is a required property here, but this section can accept additional via options.

    Attributes
    ----------
    frequencyPenalty : Optional[float]
        The frequency penalty to apply to the model's output
    maxOutputTokens : Optional[int]
        The maximum number of tokens to generate in the output
    presencePenalty : Optional[float]
        The presence penalty to apply to the model's output
    seed : Optional[int]
        A random seed for deterministic output
    temperature : Optional[float]
        The temperature to use for sampling
    topK : Optional[int]
        The top-K sampling value
    topP : Optional[float]
        The top-P sampling value
    stopSequences : list[str]
        Stop sequences to end generation
    allowMultipleToolCalls : Optional[bool]
        Whether to allow multiple tool calls in a single response
    additionalProperties : Optional[dict[str, Any]]
        Additional custom properties for model options
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    frequencyPenalty: Optional[float] = None
    maxOutputTokens: Optional[int] = None
    presencePenalty: Optional[float] = None
    seed: Optional[int] = None
    temperature: Optional[float] = None
    topK: Optional[int] = None
    topP: Optional[float] = None
    stopSequences: list[str] = field(default_factory=list)
    allowMultipleToolCalls: Optional[bool] = None
    additionalProperties: Optional[dict[str, Any]] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ModelOptions":
        """Load a ModelOptions instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ModelOptions: The loaded ModelOptions instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ModelOptions: {data}")

        # create new instance
        instance = ModelOptions()

        if data is not None and "frequencyPenalty" in data:
            instance.frequencyPenalty = data["frequencyPenalty"]
        if data is not None and "maxOutputTokens" in data:
            instance.maxOutputTokens = data["maxOutputTokens"]
        if data is not None and "presencePenalty" in data:
            instance.presencePenalty = data["presencePenalty"]
        if data is not None and "seed" in data:
            instance.seed = data["seed"]
        if data is not None and "temperature" in data:
            instance.temperature = data["temperature"]
        if data is not None and "topK" in data:
            instance.topK = data["topK"]
        if data is not None and "topP" in data:
            instance.topP = data["topP"]
        if data is not None and "stopSequences" in data:
            instance.stopSequences = data["stopSequences"]
        if data is not None and "allowMultipleToolCalls" in data:
            instance.allowMultipleToolCalls = data["allowMultipleToolCalls"]
        if data is not None and "additionalProperties" in data:
            instance.additionalProperties = data["additionalProperties"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ModelOptions instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.frequencyPenalty is not None:
            result["frequencyPenalty"] = obj.frequencyPenalty
        if obj.maxOutputTokens is not None:
            result["maxOutputTokens"] = obj.maxOutputTokens
        if obj.presencePenalty is not None:
            result["presencePenalty"] = obj.presencePenalty
        if obj.seed is not None:
            result["seed"] = obj.seed
        if obj.temperature is not None:
            result["temperature"] = obj.temperature
        if obj.topK is not None:
            result["topK"] = obj.topK
        if obj.topP is not None:
            result["topP"] = obj.topP
        if obj.stopSequences is not None:
            result["stopSequences"] = obj.stopSequences
        if obj.allowMultipleToolCalls is not None:
            result["allowMultipleToolCalls"] = obj.allowMultipleToolCalls
        if obj.additionalProperties is not None:
            result["additionalProperties"] = obj.additionalProperties

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ModelOptions instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ModelOptions instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext
from ._Format import Format
from ._Parser import Parser


@dataclass
class Template:
    """Template model for defining prompt templates.

    This model specifies the rendering engine used for slot filling prompts,
    the parser used to process the rendered template into API-compatible format,
    and additional options for the template engine.

    It allows for the creation of reusable templates that can be filled with dynamic data
    and processed to generate prompts for AI models.

    Attributes
    ----------
    format : Format
        Template rendering engine used for slot filling prompts (e.g., mustache, jinja2)
    parser : Parser
        Parser used to process the rendered template into API-compatible format
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    format: Format = field(default_factory=Format)
    parser: Parser = field(default_factory=Parser)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Template":
        """Load a Template instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Template: The loaded Template instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Template: {data}")

        # create new instance
        instance = Template()

        if data is not None and "format" in data:
            instance.format = Format.load(data["format"], context)
        if data is not None and "parser" in data:
            instance.parser = Parser.load(data["parser"], context)
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Template instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.format is not None:
            result["format"] = obj.format.save(context)
        if obj.parser is not None:
            result["parser"] = obj.parser.save(context)

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Template instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Template instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class Binding:
    """Represents a binding between an input property and a tool parameter.

    Attributes
    ----------
    name : str
        Name of the binding
    input : str
        The input property that will be bound to the tool parameter argument
    """

    _shorthand_property: ClassVar[Optional[str]] = "input"

    name: str = field(default="")
    input: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Binding":
        """Load a Binding instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Binding: The loaded Binding instance.

        """

        if context is not None:
            data = context.process_input(data)

        # handle alternate representations
        if isinstance(data, str):
            data = {"input": data}

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Binding: {data}")

        # create new instance
        instance = Binding()

        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "input" in data:
            instance.input = data["input"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Binding instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.name is not None:
            result["name"] = obj.name
        if obj.input is not None:
            result["input"] = obj.input

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Binding instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Binding instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext
from ._Connection import Connection
from ._ModelOptions import ModelOptions


@dataclass
class Model:
    """Model for defining the structure and behavior of AI agents.
    This model includes properties for specifying the model's provider, connection details, and various options.
    It allows for flexible configuration of AI models to suit different use cases and requirements.

    Attributes
    ----------
    id : str
        The unique identifier of the model - can be used as the single property shorthand
    provider : Optional[str]
        The provider of the model (e.g., 'openai', 'azure', 'anthropic')
    apiType : Optional[str]
        The type of API to use for the model (e.g., 'chat', 'response', etc.)
    connection : Optional[Connection]
        The connection configuration for the model
    options : Optional[ModelOptions]
        Additional options for the model
    """

    _shorthand_property: ClassVar[Optional[str]] = "id"

    id: str = field(default="")
    provider: Optional[str] = None
    apiType: Optional[str] = None
    connection: Optional[Connection] = None
    options: Optional[ModelOptions] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Model":
        """Load a Model instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Model: The loaded Model instance.

        """

        if context is not None:
            data = context.process_input(data)

        # handle alternate representations
        if isinstance(data, str):
            data = {"id": data}

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Model: {data}")

        # create new instance
        instance = Model()

        if data is not None and "id" in data:
            instance.id = data["id"]
        if data is not None and "provider" in data:
            instance.provider = data["provider"]
        if data is not None and "apiType" in data:
            instance.apiType = data["apiType"]
        if data is not None and "connection" in data:
            instance.connection = Connection.load(data["connection"], context)
        if data is not None and "options" in data:
            instance.options = ModelOptions.load(data["options"], context)
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Model instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.id is not None:
            result["id"] = obj.id
        if obj.provider is not None:
            result["provider"] = obj.provider
        if obj.apiType is not None:
            result["apiType"] = obj.apiType
        if obj.connection is not None:
            result["connection"] = obj.connection.save(context)
        if obj.options is not None:
            result["options"] = obj.options.save(context)

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Model instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Model instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class Property:
    """Represents a single property.

    - This model defines the structure of properties that can be used in prompts,
    including their type, description, whether they are required, and other attributes.
    - It allows for the definition of dynamic inputs that can be filled with data
    and processed to generate prompts for AI models.

    Attributes
    ----------
    name : str
        Name of the property
    kind : str
        The data type of the input property
    description : Optional[str]
        A short description of the input property
    required : Optional[bool]
        Whether the property is required
    default : Optional[Any]
        The default value of the property - this represents the default value if none is provided
    example : Optional[Any]
        Example value used for either initialization or tooling
    enumValues : list[Any]
        Allowed enumeration values for the property
    """

    _shorthand_property: ClassVar[Optional[str]] = "example"

    name: str = field(default="")
    kind: str = field(default="")
    description: Optional[str] = None
    required: Optional[bool] = None
    default: Optional[Any] = None
    example: Optional[Any] = None
    enumValues: list[Any] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Property":
        """Load a Property instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Property: The loaded Property instance.

        """

        if context is not None:
            data = context.process_input(data)

        # handle alternate representations
        if isinstance(data, bool):
            data = {"kind": "boolean", "example": data}
        if isinstance(data, float):
            data = {"kind": "float", "example": data}
        if isinstance(data, int):
            data = {"kind": "integer", "example": data}
        if isinstance(data, str):
            data = {"kind": "string", "example": data}

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Property: {data}")

        # load polymorphic Property instance
        instance = Property.load_kind(data, context)

        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "description" in data:
            instance.description = data["description"]
        if data is not None and "required" in data:
            instance.required = data["required"]
        if data is not None and "default" in data:
            instance.default = data["default"]
        if data is not None and "example" in data:
            instance.example = data["example"]
        if data is not None and "enumValues" in data:
            instance.enumValues = data["enumValues"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_kind(data: dict, context: Optional[LoadContext]) -> "Property":
        # load polymorphic Property instance
        if data is not None and "kind" in data:
            discriminator_value = str(data["kind"]).lower()
            if discriminator_value == "array":
                return ArrayProperty.load(data, context)
            elif discriminator_value == "object":
                return ObjectProperty.load(data, context)

            else:

                # create new instance (stop recursion)
                return Property()

        else:

            # create new instance
            return Property()

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Property instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.name is not None:
            result["name"] = obj.name
        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.description is not None:
            result["description"] = obj.description
        if obj.required is not None:
            result["required"] = obj.required
        if obj.default is not None:
            result["default"] = obj.default
        if obj.example is not None:
            result["example"] = obj.example
        if obj.enumValues is not None:
            result["enumValues"] = obj.enumValues

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Property instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Property instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ArrayProperty(Property):
    """Represents an array property.
    This extends the base Property model to represent an array of items.

    Attributes
    ----------
    kind : str

    items : Property
        The type of items contained in the array
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="array")
    items: Property = field(default_factory=Property)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ArrayProperty":
        """Load a ArrayProperty instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ArrayProperty: The loaded ArrayProperty instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ArrayProperty: {data}")

        # create new instance
        instance = ArrayProperty()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "items" in data:
            instance.items = Property.load(data["items"], context)
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ArrayProperty instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.items is not None:
            result["items"] = obj.items.save(context)

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ArrayProperty instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ArrayProperty instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ObjectProperty(Property):
    """Represents an object property.
    This extends the base Property model to represent a structured object.

    Attributes
    ----------
    kind : str

    properties : list[Property]
        The properties contained in the object
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="object")
    properties: list[Property] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ObjectProperty":
        """Load a ObjectProperty instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ObjectProperty: The loaded ObjectProperty instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ObjectProperty: {data}")

        # create new instance
        instance = ObjectProperty()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "properties" in data:
            instance.properties = ObjectProperty.load_properties(
                data["properties"], context
            )
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_properties(
        data: dict | list, context: Optional[LoadContext]
    ) -> list[Property]:
        if isinstance(data, dict):
            # convert simple named properties to list of Property
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "": v})
            data = result
        return [Property.load(item, context) for item in data]

    @staticmethod
    def save_properties(
        items: list[Property], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ObjectProperty instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.properties is not None:
            result["properties"] = ObjectProperty.save_properties(
                obj.properties, context
            )

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ObjectProperty instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ObjectProperty instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class ProtocolVersionRecord:
    """

    Attributes
    ----------
    protocol : str
        The protocol type.
    version : str
        The version string for the protocol, e.g. 'v0.1.1'.
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    protocol: str = field(default="")
    version: str = field(default="")

    @staticmethod
    def load(
        data: Any, context: Optional[LoadContext] = None
    ) -> "ProtocolVersionRecord":
        """Load a ProtocolVersionRecord instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ProtocolVersionRecord: The loaded ProtocolVersionRecord instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ProtocolVersionRecord: {data}")

        # create new instance
        instance = ProtocolVersionRecord()

        if data is not None and "protocol" in data:
            instance.protocol = data["protocol"]
        if data is not None and "version" in data:
            instance.version = data["version"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ProtocolVersionRecord instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.protocol is not None:
            result["protocol"] = obj.protocol
        if obj.version is not None:
            result["version"] = obj.version

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ProtocolVersionRecord instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ProtocolVersionRecord instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.

import json
from dataclasses import dataclass
from typing import Any, Callable, Optional

import yaml


@dataclass
class LoadContext:
    """
    Context for customizing the loading process of agent definitions.

    Provides hooks for pre-processing input data before parsing and
    post-processing output data after instantiation.
    """

    pre_process: Optional[Callable[[dict[str, Any]], dict[str, Any]]] = None
    """Optional callback to transform input data before parsing."""

    post_process: Optional[Callable[[Any], Any]] = None
    """Optional callback to transform the result after instantiation."""

    def process_input(self, data: dict[str, Any]) -> dict[str, Any]:
        """
        Apply pre-processing to input data if a pre_process callback is set.

        Args:
            data: The raw input dictionary to process.

        Returns:
            The processed dictionary, or the original if no callback is set.
        """
        if self.pre_process is not None:
            return self.pre_process(data)
        return data

    def process_output(self, result: Any) -> Any:
        """
        Apply post-processing to the result if a post_process callback is set.

        Args:
            result: The instantiated object to process.

        Returns:
            The processed result, or the original if no callback is set.
        """
        if self.post_process is not None:
            return self.post_process(result)
        return result


@dataclass
class SaveContext:
    """
    Context for customizing the serialization process of agent definitions.

    Provides hooks for pre-processing the object before serialization and
    post-processing the dictionary after serialization.
    """

    pre_save: Optional[Callable[[Any], Any]] = None
    """Optional callback to transform the object before serialization."""

    post_save: Optional[Callable[[dict[str, Any]], dict[str, Any]]] = None
    """Optional callback to transform the dictionary after serialization."""

    collection_format: str = "object"
    """Output format for collections: 'object' (name as key) or 'array' (list of dicts)."""

    use_shorthand: bool = True
    """Use shorthand scalar representation when possible (e.g., {"myTool": "function"})."""

    def process_object(self, obj: Any) -> Any:
        """
        Apply pre-processing to the object if a pre_save callback is set.

        Args:
            obj: The object to process before serialization.

        Returns:
            The processed object, or the original if no callback is set.
        """
        if self.pre_save is not None:
            return self.pre_save(obj)
        return obj

    def process_dict(self, data: dict[str, Any]) -> dict[str, Any]:
        """
        Apply post-processing to the dictionary if a post_save callback is set.

        Args:
            data: The serialized dictionary to process.

        Returns:
            The processed dictionary, or the original if no callback is set.
        """
        if self.post_save is not None:
            return self.post_save(data)
        return data

    def to_yaml(self, data: dict[str, Any]) -> str:
        """
        Convert the dictionary to a YAML string.

        Args:
            data: The dictionary to convert.

        Returns:
            The YAML string representation.
        """
        return yaml.dump(data, default_flow_style=False, sort_keys=False)

    def to_json(self, data: dict[str, Any], indent: int = 2) -> str:
        """
        Convert the dictionary to a JSON string.

        Args:
            data: The dictionary to convert.
            indent: Number of spaces for indentation.

        Returns:
            The JSON string representation.
        """
        return json.dumps(data, indent=indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext
from ._Property import Property


@dataclass
class PropertySchema:
    """Definition for the property schema of a model.
    This includes the properties and example records.

    Attributes
    ----------
    examples : Optional[dict[str, Any]]
        Example records for the input schema
    strict : Optional[bool]
        Whether the input schema is strict - if true, only the defined properties are allowed
    properties : list[Property]
        The input properties for the schema
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    examples: Optional[dict[str, Any]] = field(default_factory=list)
    strict: Optional[bool] = None
    properties: list[Property] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "PropertySchema":
        """Load a PropertySchema instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            PropertySchema: The loaded PropertySchema instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for PropertySchema: {data}")

        # create new instance
        instance = PropertySchema()

        if data is not None and "examples" in data:
            instance.examples = data["examples"]
        if data is not None and "strict" in data:
            instance.strict = data["strict"]
        if data is not None and "properties" in data:
            instance.properties = PropertySchema.load_properties(
                data["properties"], context
            )
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_properties(
        data: dict | list, context: Optional[LoadContext]
    ) -> list[Property]:
        if isinstance(data, dict):
            # convert simple named properties to list of Property
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "kind": v})
            data = result
        return [Property.load(item, context) for item in data]

    @staticmethod
    def save_properties(
        items: list[Property], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the PropertySchema instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.examples is not None:
            result["examples"] = obj.examples
        if obj.strict is not None:
            result["strict"] = obj.strict
        if obj.properties is not None:
            result["properties"] = PropertySchema.save_properties(
                obj.properties, context
            )

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the PropertySchema instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the PropertySchema instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class EnvironmentVariable:
    """Definition for an environment variable used in containerized agents.

    Attributes
    ----------
    name : str
        Name of the environment variable
    value : str
        Environment variable resolution
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    name: str = field(default="")
    value: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "EnvironmentVariable":
        """Load a EnvironmentVariable instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            EnvironmentVariable: The loaded EnvironmentVariable instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for EnvironmentVariable: {data}")

        # create new instance
        instance = EnvironmentVariable()

        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "value" in data:
            instance.value = data["value"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the EnvironmentVariable instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.name is not None:
            result["name"] = obj.name
        if obj.value is not None:
            result["value"] = obj.value

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the EnvironmentVariable instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the EnvironmentVariable instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from abc import ABC
from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class McpServerApprovalMode(ABC):
    """The approval mode for MCP server tools.

    Attributes
    ----------
    kind : str
        The kind identifier for string approval modes
    """

    _shorthand_property: ClassVar[Optional[str]] = "kind"

    kind: str = field(default="")

    @staticmethod
    def load(
        data: Any, context: Optional[LoadContext] = None
    ) -> "McpServerApprovalMode":
        """Load a McpServerApprovalMode instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            McpServerApprovalMode: The loaded McpServerApprovalMode instance.

        """

        if context is not None:
            data = context.process_input(data)

        # handle alternate representations
        if isinstance(data, str):
            data = {"kind": data}

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for McpServerApprovalMode: {data}")

        # load polymorphic McpServerApprovalMode instance
        instance = McpServerApprovalMode.load_kind(data, context)

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_kind(
        data: dict, context: Optional[LoadContext]
    ) -> "McpServerApprovalMode":
        # load polymorphic McpServerApprovalMode instance
        if data is not None and "kind" in data:
            discriminator_value = str(data["kind"]).lower()
            if discriminator_value == "always":
                return McpServerToolAlwaysRequireApprovalMode.load(data, context)
            elif discriminator_value == "never":
                return McpServerToolNeverRequireApprovalMode.load(data, context)
            elif discriminator_value == "specify":
                return McpServerToolSpecifyApprovalMode.load(data, context)

            else:
                raise ValueError(
                    f"Unknown McpServerApprovalMode discriminator value: {discriminator_value}"
                )
        else:

            raise ValueError(
                "Missing McpServerApprovalMode discriminator property: 'kind'"
            )

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the McpServerApprovalMode instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.kind is not None:
            result["kind"] = obj.kind

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the McpServerApprovalMode instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the McpServerApprovalMode instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class McpServerToolAlwaysRequireApprovalMode(McpServerApprovalMode):
    """

    Attributes
    ----------
    kind : str
        The kind identifier for always approval mode
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="always")

    @staticmethod
    def load(
        data: Any, context: Optional[LoadContext] = None
    ) -> "McpServerToolAlwaysRequireApprovalMode":
        """Load a McpServerToolAlwaysRequireApprovalMode instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            McpServerToolAlwaysRequireApprovalMode: The loaded McpServerToolAlwaysRequireApprovalMode instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(
                f"Invalid data for McpServerToolAlwaysRequireApprovalMode: {data}"
            )

        # create new instance
        instance = McpServerToolAlwaysRequireApprovalMode()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the McpServerToolAlwaysRequireApprovalMode instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the McpServerToolAlwaysRequireApprovalMode instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the McpServerToolAlwaysRequireApprovalMode instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class McpServerToolNeverRequireApprovalMode(McpServerApprovalMode):
    """

    Attributes
    ----------
    kind : str
        The kind identifier for never approval mode
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="never")

    @staticmethod
    def load(
        data: Any, context: Optional[LoadContext] = None
    ) -> "McpServerToolNeverRequireApprovalMode":
        """Load a McpServerToolNeverRequireApprovalMode instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            McpServerToolNeverRequireApprovalMode: The loaded McpServerToolNeverRequireApprovalMode instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(
                f"Invalid data for McpServerToolNeverRequireApprovalMode: {data}"
            )

        # create new instance
        instance = McpServerToolNeverRequireApprovalMode()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the McpServerToolNeverRequireApprovalMode instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the McpServerToolNeverRequireApprovalMode instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the McpServerToolNeverRequireApprovalMode instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class McpServerToolSpecifyApprovalMode(McpServerApprovalMode):
    """

    Attributes
    ----------
    kind : str
        The kind identifier for specify approval mode
    alwaysRequireApprovalTools : list[str]
        List of tools that always require approval
    neverRequireApprovalTools : list[str]
        List of tools that never require approval
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="specify")
    alwaysRequireApprovalTools: list[str] = field(default_factory=list)
    neverRequireApprovalTools: list[str] = field(default_factory=list)

    @staticmethod
    def load(
        data: Any, context: Optional[LoadContext] = None
    ) -> "McpServerToolSpecifyApprovalMode":
        """Load a McpServerToolSpecifyApprovalMode instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            McpServerToolSpecifyApprovalMode: The loaded McpServerToolSpecifyApprovalMode instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(
                f"Invalid data for McpServerToolSpecifyApprovalMode: {data}"
            )

        # create new instance
        instance = McpServerToolSpecifyApprovalMode()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "alwaysRequireApprovalTools" in data:
            instance.alwaysRequireApprovalTools = data["alwaysRequireApprovalTools"]
        if data is not None and "neverRequireApprovalTools" in data:
            instance.neverRequireApprovalTools = data["neverRequireApprovalTools"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the McpServerToolSpecifyApprovalMode instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.alwaysRequireApprovalTools is not None:
            result["alwaysRequireApprovalTools"] = obj.alwaysRequireApprovalTools
        if obj.neverRequireApprovalTools is not None:
            result["neverRequireApprovalTools"] = obj.neverRequireApprovalTools

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the McpServerToolSpecifyApprovalMode instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the McpServerToolSpecifyApprovalMode instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from abc import ABC
from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext
from ._EnvironmentVariable import EnvironmentVariable
from ._Model import Model
from ._PropertySchema import PropertySchema
from ._ProtocolVersionRecord import ProtocolVersionRecord
from ._Template import Template
from ._Tool import Tool


@dataclass
class AgentDefinition(ABC):
    """The following is a specification for defining AI agents with structured metadata, inputs, outputs, tools, and templates.
    It provides a way to create reusable and composable AI agents that can be executed with specific configurations.
    The specification includes metadata about the agent, model configuration, input parameters, expected outputs,
    available tools, and template configurations for prompt rendering.

    Attributes
    ----------
    kind : str
        Kind represented by the document
    name : str
        Human-readable name of the agent
    displayName : Optional[str]
        Display name of the agent for UI purposes
    description : Optional[str]
        Description of the agent's capabilities and purpose
    metadata : Optional[dict[str, Any]]
        Additional metadata including authors, tags, and other arbitrary properties
    inputSchema : Optional[PropertySchema]
        Input parameters that participate in template rendering
    outputSchema : Optional[PropertySchema]
        Expected output format and structure from the agent
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="")
    name: str = field(default="")
    displayName: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[dict[str, Any]] = None
    inputSchema: Optional[PropertySchema] = None
    outputSchema: Optional[PropertySchema] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "AgentDefinition":
        """Load a AgentDefinition instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            AgentDefinition: The loaded AgentDefinition instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for AgentDefinition: {data}")

        # load polymorphic AgentDefinition instance
        instance = AgentDefinition.load_kind(data, context)

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "displayName" in data:
            instance.displayName = data["displayName"]
        if data is not None and "description" in data:
            instance.description = data["description"]
        if data is not None and "metadata" in data:
            instance.metadata = data["metadata"]
        if data is not None and "inputSchema" in data:
            instance.inputSchema = PropertySchema.load(data["inputSchema"], context)
        if data is not None and "outputSchema" in data:
            instance.outputSchema = PropertySchema.load(data["outputSchema"], context)
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_kind(data: dict, context: Optional[LoadContext]) -> "AgentDefinition":
        # load polymorphic AgentDefinition instance
        if data is not None and "kind" in data:
            discriminator_value = str(data["kind"]).lower()
            if discriminator_value == "prompt":
                return PromptAgent.load(data, context)
            elif discriminator_value == "workflow":
                return Workflow.load(data, context)
            elif discriminator_value == "hosted":
                return ContainerAgent.load(data, context)

            else:
                raise ValueError(
                    f"Unknown AgentDefinition discriminator value: {discriminator_value}"
                )
        else:

            raise ValueError("Missing AgentDefinition discriminator property: 'kind'")

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the AgentDefinition instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.name is not None:
            result["name"] = obj.name
        if obj.displayName is not None:
            result["displayName"] = obj.displayName
        if obj.description is not None:
            result["description"] = obj.description
        if obj.metadata is not None:
            result["metadata"] = obj.metadata
        if obj.inputSchema is not None:
            result["inputSchema"] = obj.inputSchema.save(context)
        if obj.outputSchema is not None:
            result["outputSchema"] = obj.outputSchema.save(context)

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the AgentDefinition instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the AgentDefinition instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class PromptAgent(AgentDefinition):
    """Prompt based agent definition. Used to create agents that can be executed directly.
    These agents can leverage tools, input parameters, and templates to generate responses.
    They are designed to be straightforward and easy to use for various applications.

    Attributes
    ----------
    kind : str
        Type of agent, e.g., 'prompt'
    model : Model
        Primary AI model configuration for the agent
    tools : list[Tool]
        Tools available to the agent for extended functionality
    template : Optional[Template]
        Template configuration for prompt rendering
    instructions : Optional[str]
        Give your agent clear directions on what to do and how to do it. Include specific tasks, their order, and any special instructions like tone or engagement style. (can use this for a pure yaml declaration or as content in the markdown format)
    additionalInstructions : Optional[str]
        Additional instructions or context for the agent, can be used to provide extra guidance (can use this for a pure yaml declaration)
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="prompt")
    model: Model = field(default_factory=Model)
    tools: list[Tool] = field(default_factory=list)
    template: Optional[Template] = None
    instructions: Optional[str] = None
    additionalInstructions: Optional[str] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "PromptAgent":
        """Load a PromptAgent instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            PromptAgent: The loaded PromptAgent instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for PromptAgent: {data}")

        # create new instance
        instance = PromptAgent()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "model" in data:
            instance.model = Model.load(data["model"], context)
        if data is not None and "tools" in data:
            instance.tools = PromptAgent.load_tools(data["tools"], context)
        if data is not None and "template" in data:
            instance.template = Template.load(data["template"], context)
        if data is not None and "instructions" in data:
            instance.instructions = data["instructions"]
        if data is not None and "additionalInstructions" in data:
            instance.additionalInstructions = data["additionalInstructions"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_tools(data: dict | list, context: Optional[LoadContext]) -> list[Tool]:
        if isinstance(data, dict):
            # convert simple named tools to list of Tool
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "kind": v})
            data = result
        return [Tool.load(item, context) for item in data]

    @staticmethod
    def save_tools(
        items: list[Tool], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the PromptAgent instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.model is not None:
            result["model"] = obj.model.save(context)
        if obj.tools is not None:
            result["tools"] = PromptAgent.save_tools(obj.tools, context)
        if obj.template is not None:
            result["template"] = obj.template.save(context)
        if obj.instructions is not None:
            result["instructions"] = obj.instructions
        if obj.additionalInstructions is not None:
            result["additionalInstructions"] = obj.additionalInstructions

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the PromptAgent instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the PromptAgent instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class Workflow(AgentDefinition):
    """A workflow agent that can orchestrate multiple steps and actions.
    This agent type is designed to handle complex workflows that may involve
    multiple tools, models, and decision points.

    The workflow agent can be configured with a series of steps that define
    the flow of execution, including conditional logic and parallel processing.
    This allows for the creation of sophisticated AI-driven processes that can
    adapt to various scenarios and requirements.

    Note: The detailed structure of the workflow steps and actions is not defined here
    and would need to be implemented based on specific use cases and requirements.

    Attributes
    ----------
    kind : str
        Type of agent, e.g., 'workflow'
    trigger : Optional[dict[str, Any]]
        The steps that make up the workflow
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="workflow")
    trigger: Optional[dict[str, Any]] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Workflow":
        """Load a Workflow instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Workflow: The loaded Workflow instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Workflow: {data}")

        # create new instance
        instance = Workflow()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "trigger" in data:
            instance.trigger = data["trigger"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Workflow instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.trigger is not None:
            result["trigger"] = obj.trigger

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Workflow instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Workflow instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ContainerAgent(AgentDefinition):
    """This represents a container based agent hosted by the provider/publisher.
    The intent is to represent a container application that the user wants to run
    in a hosted environment that the provider manages.

    Attributes
    ----------
    kind : str
        Type of agent, e.g., 'hosted'
    protocols : list[ProtocolVersionRecord]
        Protocol used by the containerized agent
    environmentVariables : list[EnvironmentVariable]
        Environment variables to set in the container
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="hosted")
    protocols: list[ProtocolVersionRecord] = field(default_factory=list)
    environmentVariables: list[EnvironmentVariable] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ContainerAgent":
        """Load a ContainerAgent instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ContainerAgent: The loaded ContainerAgent instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ContainerAgent: {data}")

        # create new instance
        instance = ContainerAgent()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "protocols" in data:
            instance.protocols = ContainerAgent.load_protocols(
                data["protocols"], context
            )
        if data is not None and "environmentVariables" in data:
            instance.environmentVariables = ContainerAgent.load_environmentVariables(
                data["environmentVariables"], context
            )
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_protocols(
        data: dict | list, context: Optional[LoadContext]
    ) -> list[ProtocolVersionRecord]:
        if isinstance(data, dict):
            # convert simple named protocols to list of ProtocolVersionRecord
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "protocol": v})
            data = result
        return [ProtocolVersionRecord.load(item, context) for item in data]

    @staticmethod
    def save_protocols(
        items: list[ProtocolVersionRecord], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    @staticmethod
    def load_environmentVariables(
        data: dict | list, context: Optional[LoadContext]
    ) -> list[EnvironmentVariable]:
        if isinstance(data, dict):
            # convert simple named environmentVariables to list of EnvironmentVariable
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "value": v})
            data = result
        return [EnvironmentVariable.load(item, context) for item in data]

    @staticmethod
    def save_environmentVariables(
        items: list[EnvironmentVariable], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ContainerAgent instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.protocols is not None:
            result["protocols"] = ContainerAgent.save_protocols(obj.protocols, context)
        if obj.environmentVariables is not None:
            result["environmentVariables"] = ContainerAgent.save_environmentVariables(
                obj.environmentVariables, context
            )

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ContainerAgent instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ContainerAgent instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from abc import ABC
from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class Resource(ABC):
    """Represents a resource required by the agent
    Resources can include databases, APIs, or other external systems
    that the agent needs to interact with to perform its tasks

    Attributes
    ----------
    name : str
        Name of the resource
    kind : str
        The kind of resource (e.g., model, tool)
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    name: str = field(default="")
    kind: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Resource":
        """Load a Resource instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Resource: The loaded Resource instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Resource: {data}")

        # load polymorphic Resource instance
        instance = Resource.load_kind(data, context)

        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_kind(data: dict, context: Optional[LoadContext]) -> "Resource":
        # load polymorphic Resource instance
        if data is not None and "kind" in data:
            discriminator_value = str(data["kind"]).lower()
            if discriminator_value == "model":
                return ModelResource.load(data, context)
            elif discriminator_value == "tool":
                return ToolResource.load(data, context)

            else:
                raise ValueError(
                    f"Unknown Resource discriminator value: {discriminator_value}"
                )
        else:

            raise ValueError("Missing Resource discriminator property: 'kind'")

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Resource instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.name is not None:
            result["name"] = obj.name
        if obj.kind is not None:
            result["kind"] = obj.kind

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Resource instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Resource instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ModelResource(Resource):
    """Represents a model resource required by the agent

    Attributes
    ----------
    kind : str
        The kind identifier for model resources
    id : str
        The unique identifier of the model resource
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="model")
    id: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ModelResource":
        """Load a ModelResource instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ModelResource: The loaded ModelResource instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ModelResource: {data}")

        # create new instance
        instance = ModelResource()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "id" in data:
            instance.id = data["id"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ModelResource instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.id is not None:
            result["id"] = obj.id

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ModelResource instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ModelResource instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ToolResource(Resource):
    """Represents a tool resource required by the agent

    Attributes
    ----------
    kind : str
        The kind identifier for tool resources
    id : str
        The unique identifier of the tool resource
    options : dict[str, Any]
        Configuration options for the tool resource
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="tool")
    id: str = field(default="")
    options: dict[str, Any] = field(default_factory=dict)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ToolResource":
        """Load a ToolResource instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ToolResource: The loaded ToolResource instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ToolResource: {data}")

        # create new instance
        instance = ToolResource()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "id" in data:
            instance.id = data["id"]
        if data is not None and "options" in data:
            instance.options = data["options"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ToolResource instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.id is not None:
            result["id"] = obj.id
        if obj.options is not None:
            result["options"] = obj.options

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ToolResource instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ToolResource instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.

from ._context import LoadContext, SaveContext

from ._AgentDefinition import AgentDefinition, PromptAgent, Workflow, ContainerAgent


from ._Connection import (
    Connection,
    ReferenceConnection,
    RemoteConnection,
    ApiKeyConnection,
    AnonymousConnection,
)


from ._ModelOptions import ModelOptions


from ._Model import Model


from ._Binding import Binding


from ._Tool import (
    Tool,
    FunctionTool,
    CustomTool,
    WebSearchTool,
    FileSearchTool,
    McpTool,
    OpenApiTool,
    CodeInterpreterTool,
)


from ._Property import Property, ArrayProperty, ObjectProperty


from ._PropertySchema import PropertySchema


from ._McpServerApprovalMode import (
    McpServerApprovalMode,
    McpServerToolAlwaysRequireApprovalMode,
    McpServerToolNeverRequireApprovalMode,
    McpServerToolSpecifyApprovalMode,
)


from ._Format import Format


from ._Parser import Parser


from ._Template import Template


from ._ProtocolVersionRecord import ProtocolVersionRecord


from ._EnvironmentVariable import EnvironmentVariable


from ._Resource import Resource, ModelResource, ToolResource


from ._AgentManifest import AgentManifest


__all__ = [
    "LoadContext",
    "SaveContext",
    "AgentDefinition",
    "Connection",
    "ReferenceConnection",
    "RemoteConnection",
    "ApiKeyConnection",
    "AnonymousConnection",
    "ModelOptions",
    "Model",
    "Binding",
    "Tool",
    "Property",
    "ObjectProperty",
    "ArrayProperty",
    "PropertySchema",
    "FunctionTool",
    "CustomTool",
    "WebSearchTool",
    "FileSearchTool",
    "McpServerApprovalMode",
    "McpServerToolAlwaysRequireApprovalMode",
    "McpServerToolNeverRequireApprovalMode",
    "McpServerToolSpecifyApprovalMode",
    "McpTool",
    "OpenApiTool",
    "CodeInterpreterTool",
    "Format",
    "Parser",
    "Template",
    "PromptAgent",
    "Workflow",
    "ProtocolVersionRecord",
    "EnvironmentVariable",
    "ContainerAgent",
    "Resource",
    "ModelResource",
    "ToolResource",
    "AgentManifest",
]



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class Parser:
    """Template parser definition

    Attributes
    ----------
    kind : str
        Parser used to process the rendered template into API-compatible format
    options : Optional[dict[str, Any]]
        Options for the parser
    """

    _shorthand_property: ClassVar[Optional[str]] = "kind"

    kind: str = field(default="*")
    options: Optional[dict[str, Any]] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Parser":
        """Load a Parser instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Parser: The loaded Parser instance.

        """

        if context is not None:
            data = context.process_input(data)

        # handle alternate representations
        if isinstance(data, str):
            data = {"kind": data}

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Parser: {data}")

        # create new instance
        instance = Parser()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "options" in data:
            instance.options = data["options"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Parser instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.options is not None:
            result["options"] = obj.options

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Parser instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Parser instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from abc import ABC
from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext
from ._Binding import Binding
from ._Connection import Connection
from ._McpServerApprovalMode import McpServerApprovalMode
from ._PropertySchema import PropertySchema


@dataclass
class Tool(ABC):
    """Represents a tool that can be used in prompts.

    Attributes
    ----------
    name : str
        Name of the tool. If a function tool, this is the function name, otherwise it is the type
    kind : str
        The kind identifier for the tool
    description : Optional[str]
        A short description of the tool for metadata purposes
    bindings : list[Binding]
        Tool argument bindings to input properties
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    name: str = field(default="")
    kind: str = field(default="")
    description: Optional[str] = None
    bindings: list[Binding] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Tool":
        """Load a Tool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Tool: The loaded Tool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Tool: {data}")

        # load polymorphic Tool instance
        instance = Tool.load_kind(data, context)

        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "description" in data:
            instance.description = data["description"]
        if data is not None and "bindings" in data:
            instance.bindings = Tool.load_bindings(data["bindings"], context)
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_bindings(
        data: dict | list, context: Optional[LoadContext]
    ) -> list[Binding]:
        if isinstance(data, dict):
            # convert simple named bindings to list of Binding
            result = []
            for k, v in data.items():
                if isinstance(v, dict):
                    # value is an object, spread its properties
                    result.append({"name": k, **v})
                else:
                    # value is a scalar, use it as the primary property
                    result.append({"name": k, "input": v})
            data = result
        return [Binding.load(item, context) for item in data]

    @staticmethod
    def save_bindings(
        items: list[Binding], context: Optional[SaveContext]
    ) -> dict[str, Any] | list[dict[str, Any]]:
        if context is None:
            context = SaveContext()

        if context.collection_format == "array":
            return [item.save(context) for item in items]

        # Object format: use name as key
        result: dict[str, Any] = {}
        for item in items:
            item_data = item.save(context)
            name = item_data.pop("name", None)
            if name:
                # Check if we can use shorthand (only primary property set)
                if context.use_shorthand and hasattr(item, "_shorthand_property"):
                    shorthand_prop = item._shorthand_property
                    if (
                        shorthand_prop
                        and len(item_data) == 1
                        and shorthand_prop in item_data
                    ):
                        result[name] = item_data[shorthand_prop]
                        continue
                result[name] = item_data
            else:
                # No name, fall back to array format for this item
                if "_unnamed" not in result:
                    result["_unnamed"] = []
                result["_unnamed"].append(item_data)
        return result

    @staticmethod
    def load_kind(data: dict, context: Optional[LoadContext]) -> "Tool":
        # load polymorphic Tool instance
        if data is not None and "kind" in data:
            discriminator_value = str(data["kind"]).lower()
            if discriminator_value == "function":
                return FunctionTool.load(data, context)
            elif discriminator_value == "bing_search":
                return WebSearchTool.load(data, context)
            elif discriminator_value == "file_search":
                return FileSearchTool.load(data, context)
            elif discriminator_value == "mcp":
                return McpTool.load(data, context)
            elif discriminator_value == "openapi":
                return OpenApiTool.load(data, context)
            elif discriminator_value == "code_interpreter":
                return CodeInterpreterTool.load(data, context)

            else:

                # load default instance
                return CustomTool.load(data, context)

        else:

            raise ValueError("Missing Tool discriminator property: 'kind'")

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Tool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.name is not None:
            result["name"] = obj.name
        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.description is not None:
            result["description"] = obj.description
        if obj.bindings is not None:
            result["bindings"] = Tool.save_bindings(obj.bindings, context)

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Tool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Tool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class FunctionTool(Tool):
    """Represents a local function tool.

    Attributes
    ----------
    kind : str
        The kind identifier for function tools
    parameters : PropertySchema
        Parameters accepted by the function tool
    strict : Optional[bool]
        Indicates whether the function tool enforces strict validation on its parameters
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="function")
    parameters: PropertySchema = field(default_factory=PropertySchema)
    strict: Optional[bool] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "FunctionTool":
        """Load a FunctionTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            FunctionTool: The loaded FunctionTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for FunctionTool: {data}")

        # create new instance
        instance = FunctionTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "parameters" in data:
            instance.parameters = PropertySchema.load(data["parameters"], context)
        if data is not None and "strict" in data:
            instance.strict = data["strict"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the FunctionTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.parameters is not None:
            result["parameters"] = obj.parameters.save(context)
        if obj.strict is not None:
            result["strict"] = obj.strict

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the FunctionTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the FunctionTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class CustomTool(Tool):
    """Represents a generic server tool that runs on a server
    This tool kind is designed for operations that require server-side execution
    It may include features such as authentication, data storage, and long-running processes
    This tool kind is ideal for tasks that involve complex computations or access to secure resources
    Server tools can be used to offload heavy processing from client applications

    Attributes
    ----------
    kind : str
        The kind identifier for server tools. This is a wildcard and can represent any server tool type not explicitly defined.
    connection : Connection
        Connection configuration for the server tool
    options : dict[str, Any]
        Configuration options for the server tool
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="*")
    connection: Connection = field(default_factory=Connection)
    options: dict[str, Any] = field(default_factory=dict)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "CustomTool":
        """Load a CustomTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            CustomTool: The loaded CustomTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for CustomTool: {data}")

        # create new instance
        instance = CustomTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "connection" in data:
            instance.connection = Connection.load(data["connection"], context)
        if data is not None and "options" in data:
            instance.options = data["options"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the CustomTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.connection is not None:
            result["connection"] = obj.connection.save(context)
        if obj.options is not None:
            result["options"] = obj.options

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the CustomTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the CustomTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class WebSearchTool(Tool):
    """The Bing search tool.

    Attributes
    ----------
    kind : str
        The kind identifier for Bing search tools
    connection : Connection
        The connection configuration for the Bing search tool
    options : Optional[dict[str, Any]]
        The configuration options for the Bing search tool
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="bing_search")
    connection: Connection = field(default_factory=Connection)
    options: Optional[dict[str, Any]] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "WebSearchTool":
        """Load a WebSearchTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            WebSearchTool: The loaded WebSearchTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for WebSearchTool: {data}")

        # create new instance
        instance = WebSearchTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "connection" in data:
            instance.connection = Connection.load(data["connection"], context)
        if data is not None and "options" in data:
            instance.options = data["options"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the WebSearchTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.connection is not None:
            result["connection"] = obj.connection.save(context)
        if obj.options is not None:
            result["options"] = obj.options

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the WebSearchTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the WebSearchTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class FileSearchTool(Tool):
    """A tool for searching files.
    This tool allows an AI agent to search for files based on a query.

    Attributes
    ----------
    kind : str
        The kind identifier for file search tools
    connection : Connection
        The connection configuration for the file search tool
    vectorStoreIds : list[str]
        The IDs of the vector stores to search within.
    maximumResultCount : Optional[int]
        The maximum number of search results to return.
    ranker : Optional[str]
        File search ranker.
    scoreThreshold : Optional[float]
        Ranker search threshold.
    filters : Optional[dict[str, Any]]
        Additional filters to apply during the file search.
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="file_search")
    connection: Connection = field(default_factory=Connection)
    vectorStoreIds: list[str] = field(default_factory=list)
    maximumResultCount: Optional[int] = None
    ranker: Optional[str] = None
    scoreThreshold: Optional[float] = None
    filters: Optional[dict[str, Any]] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "FileSearchTool":
        """Load a FileSearchTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            FileSearchTool: The loaded FileSearchTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for FileSearchTool: {data}")

        # create new instance
        instance = FileSearchTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "connection" in data:
            instance.connection = Connection.load(data["connection"], context)
        if data is not None and "vectorStoreIds" in data:
            instance.vectorStoreIds = data["vectorStoreIds"]
        if data is not None and "maximumResultCount" in data:
            instance.maximumResultCount = data["maximumResultCount"]
        if data is not None and "ranker" in data:
            instance.ranker = data["ranker"]
        if data is not None and "scoreThreshold" in data:
            instance.scoreThreshold = data["scoreThreshold"]
        if data is not None and "filters" in data:
            instance.filters = data["filters"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the FileSearchTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.connection is not None:
            result["connection"] = obj.connection.save(context)
        if obj.vectorStoreIds is not None:
            result["vectorStoreIds"] = obj.vectorStoreIds
        if obj.maximumResultCount is not None:
            result["maximumResultCount"] = obj.maximumResultCount
        if obj.ranker is not None:
            result["ranker"] = obj.ranker
        if obj.scoreThreshold is not None:
            result["scoreThreshold"] = obj.scoreThreshold
        if obj.filters is not None:
            result["filters"] = obj.filters

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the FileSearchTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the FileSearchTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class McpTool(Tool):
    """The MCP Server tool.

    Attributes
    ----------
    kind : str
        The kind identifier for MCP tools
    connection : Connection
        The connection configuration for the MCP tool
    serverName : str
        The server name of the MCP tool
    serverDescription : Optional[str]
        The description of the MCP tool
    approvalMode : McpServerApprovalMode
        The approval mode for the MCP tool, either 'auto' or 'manual'
    allowedTools : list[str]
        List of allowed operations or resources for the MCP tool
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="mcp")
    connection: Connection = field(default_factory=Connection)
    serverName: str = field(default="")
    serverDescription: Optional[str] = None
    approvalMode: McpServerApprovalMode = field(default_factory=McpServerApprovalMode)
    allowedTools: list[str] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "McpTool":
        """Load a McpTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            McpTool: The loaded McpTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for McpTool: {data}")

        # create new instance
        instance = McpTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "connection" in data:
            instance.connection = Connection.load(data["connection"], context)
        if data is not None and "serverName" in data:
            instance.serverName = data["serverName"]
        if data is not None and "serverDescription" in data:
            instance.serverDescription = data["serverDescription"]
        if data is not None and "approvalMode" in data:
            instance.approvalMode = McpServerApprovalMode.load(
                data["approvalMode"], context
            )
        if data is not None and "allowedTools" in data:
            instance.allowedTools = data["allowedTools"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the McpTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.connection is not None:
            result["connection"] = obj.connection.save(context)
        if obj.serverName is not None:
            result["serverName"] = obj.serverName
        if obj.serverDescription is not None:
            result["serverDescription"] = obj.serverDescription
        if obj.approvalMode is not None:
            result["approvalMode"] = obj.approvalMode.save(context)
        if obj.allowedTools is not None:
            result["allowedTools"] = obj.allowedTools

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the McpTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the McpTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class OpenApiTool(Tool):
    """

    Attributes
    ----------
    kind : str
        The kind identifier for OpenAPI tools
    connection : Connection
        The connection configuration for the OpenAPI tool
    specification : str
        The full OpenAPI specification
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="openapi")
    connection: Connection = field(default_factory=Connection)
    specification: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "OpenApiTool":
        """Load a OpenApiTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            OpenApiTool: The loaded OpenApiTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for OpenApiTool: {data}")

        # create new instance
        instance = OpenApiTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "connection" in data:
            instance.connection = Connection.load(data["connection"], context)
        if data is not None and "specification" in data:
            instance.specification = data["specification"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the OpenApiTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.connection is not None:
            result["connection"] = obj.connection.save(context)
        if obj.specification is not None:
            result["specification"] = obj.specification

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the OpenApiTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the OpenApiTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class CodeInterpreterTool(Tool):
    """A tool for interpreting and executing code.
    This tool allows an AI agent to run code snippets and analyze data files.

    Attributes
    ----------
    kind : str
        The kind identifier for code interpreter tools
    fileIds : list[str]
        The IDs of the files to be used by the code interpreter tool.
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="code_interpreter")
    fileIds: list[str] = field(default_factory=list)

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "CodeInterpreterTool":
        """Load a CodeInterpreterTool instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            CodeInterpreterTool: The loaded CodeInterpreterTool instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for CodeInterpreterTool: {data}")

        # create new instance
        instance = CodeInterpreterTool()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "fileIds" in data:
            instance.fileIds = data["fileIds"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the CodeInterpreterTool instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.fileIds is not None:
            result["fileIds"] = obj.fileIds

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the CodeInterpreterTool instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the CodeInterpreterTool instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)



# Copyright (c) Microsoft. All rights reserved.
# WARNING: This is an auto-generated file. DO NOT EDIT THIS FILE DIRECTLY.


from abc import ABC
from dataclasses import dataclass, field
from typing import Any, ClassVar, Optional

from ._context import LoadContext, SaveContext


@dataclass
class Connection(ABC):
    """Connection configuration for AI agents.
    `provider`, `kind`, and `endpoint` are required properties here,
    but this section can accept additional via options.

    Attributes
    ----------
    kind : str
        The Authentication kind for the AI service (e.g., 'key' for API key, 'oauth' for OAuth tokens)
    authenticationMode : str
        The authority level for the connection, indicating under whose authority the connection is made (e.g., 'user', 'agent', 'system')
    usageDescription : Optional[str]
        The usage description for the connection, providing context on how this connection will be used
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="")
    authenticationMode: str = field(default="system")
    usageDescription: Optional[str] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "Connection":
        """Load a Connection instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            Connection: The loaded Connection instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for Connection: {data}")

        # load polymorphic Connection instance
        instance = Connection.load_kind(data, context)

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "authenticationMode" in data:
            instance.authenticationMode = data["authenticationMode"]
        if data is not None and "usageDescription" in data:
            instance.usageDescription = data["usageDescription"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    @staticmethod
    def load_kind(data: dict, context: Optional[LoadContext]) -> "Connection":
        # load polymorphic Connection instance
        if data is not None and "kind" in data:
            discriminator_value = str(data["kind"]).lower()
            if discriminator_value == "reference":
                return ReferenceConnection.load(data, context)
            elif discriminator_value == "remote":
                return RemoteConnection.load(data, context)
            elif discriminator_value == "key":
                return ApiKeyConnection.load(data, context)
            elif discriminator_value == "anonymous":
                return AnonymousConnection.load(data, context)

            else:
                raise ValueError(
                    f"Unknown Connection discriminator value: {discriminator_value}"
                )
        else:

            raise ValueError("Missing Connection discriminator property: 'kind'")

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the Connection instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        result: dict[str, Any] = {}

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.authenticationMode is not None:
            result["authenticationMode"] = obj.authenticationMode
        if obj.usageDescription is not None:
            result["usageDescription"] = obj.usageDescription

        if context is not None:
            result = context.process_dict(result)
        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the Connection instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the Connection instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ReferenceConnection(Connection):
    """Connection configuration for AI services using named connections.

    Attributes
    ----------
    kind : str
        The Authentication kind for the AI service (e.g., 'key' for API key, 'oauth' for OAuth tokens)
    name : str
        The name of the connection
    target : Optional[str]
        The target resource or service that this connection refers to
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="reference")
    name: str = field(default="")
    target: Optional[str] = None

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ReferenceConnection":
        """Load a ReferenceConnection instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ReferenceConnection: The loaded ReferenceConnection instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ReferenceConnection: {data}")

        # create new instance
        instance = ReferenceConnection()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "target" in data:
            instance.target = data["target"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ReferenceConnection instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.name is not None:
            result["name"] = obj.name
        if obj.target is not None:
            result["target"] = obj.target

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ReferenceConnection instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ReferenceConnection instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class RemoteConnection(Connection):
    """Connection configuration for AI services using named connections.

    Attributes
    ----------
    kind : str
        The Authentication kind for the AI service (e.g., 'key' for API key, 'oauth' for OAuth tokens)
    name : str
        The name of the connection
    endpoint : str
        The endpoint URL for the AI service
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="remote")
    name: str = field(default="")
    endpoint: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "RemoteConnection":
        """Load a RemoteConnection instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            RemoteConnection: The loaded RemoteConnection instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for RemoteConnection: {data}")

        # create new instance
        instance = RemoteConnection()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "name" in data:
            instance.name = data["name"]
        if data is not None and "endpoint" in data:
            instance.endpoint = data["endpoint"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the RemoteConnection instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.name is not None:
            result["name"] = obj.name
        if obj.endpoint is not None:
            result["endpoint"] = obj.endpoint

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the RemoteConnection instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the RemoteConnection instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class ApiKeyConnection(Connection):
    """Connection configuration for AI services using API keys.

    Attributes
    ----------
    kind : str
        The Authentication kind for the AI service (e.g., 'key' for API key, 'oauth' for OAuth tokens)
    endpoint : str
        The endpoint URL for the AI service
    apiKey : str
        The API key for authenticating with the AI service
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="key")
    endpoint: str = field(default="")
    apiKey: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "ApiKeyConnection":
        """Load a ApiKeyConnection instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            ApiKeyConnection: The loaded ApiKeyConnection instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for ApiKeyConnection: {data}")

        # create new instance
        instance = ApiKeyConnection()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "endpoint" in data:
            instance.endpoint = data["endpoint"]
        if data is not None and "apiKey" in data:
            instance.apiKey = data["apiKey"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the ApiKeyConnection instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.endpoint is not None:
            result["endpoint"] = obj.endpoint
        if obj.apiKey is not None:
            result["apiKey"] = obj.apiKey

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the ApiKeyConnection instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the ApiKeyConnection instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)


@dataclass
class AnonymousConnection(Connection):
    """

    Attributes
    ----------
    kind : str
        The Authentication kind for the AI service (e.g., 'key' for API key, 'oauth' for OAuth tokens)
    endpoint : str
        The endpoint for authenticating with the AI service
    """

    _shorthand_property: ClassVar[Optional[str]] = None

    kind: str = field(default="anonymous")
    endpoint: str = field(default="")

    @staticmethod
    def load(data: Any, context: Optional[LoadContext] = None) -> "AnonymousConnection":
        """Load a AnonymousConnection instance.
        Args:
            data (Any): The data to load the instance from.
            context (Optional[LoadContext]): Optional context with pre/post processing callbacks.
        Returns:
            AnonymousConnection: The loaded AnonymousConnection instance.

        """

        if context is not None:
            data = context.process_input(data)

        if not isinstance(data, dict):
            raise ValueError(f"Invalid data for AnonymousConnection: {data}")

        # create new instance
        instance = AnonymousConnection()

        if data is not None and "kind" in data:
            instance.kind = data["kind"]
        if data is not None and "endpoint" in data:
            instance.endpoint = data["endpoint"]
        if context is not None:
            instance = context.process_output(instance)
        return instance

    def save(self, context: Optional[SaveContext] = None) -> dict[str, Any]:
        """Save the AnonymousConnection instance to a dictionary.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            dict[str, Any]: The dictionary representation of this instance.

        """
        obj = self
        if context is not None:
            obj = context.process_object(obj)

        # Start with parent class properties
        result = super().save(context)

        if obj.kind is not None:
            result["kind"] = obj.kind
        if obj.endpoint is not None:
            result["endpoint"] = obj.endpoint

        return result

    def to_yaml(self, context: Optional[SaveContext] = None) -> str:
        """Convert the AnonymousConnection instance to a YAML string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
        Returns:
            str: The YAML string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_yaml(self.save(context))

    def to_json(self, context: Optional[SaveContext] = None, indent: int = 2) -> str:
        """Convert the AnonymousConnection instance to a JSON string.
        Args:
            context (Optional[SaveContext]): Optional context with pre/post processing callbacks.
            indent (int): Number of spaces for indentation. Defaults to 2.
        Returns:
            str: The JSON string representation of this instance.

        """
        if context is None:
            context = SaveContext()
        return context.to_json(self.save(context), indent)
